
class MultiProjectIndex(object):
    '''
    This class should be used inside the trac mod_python 
    handler 'ModPythonHandler.py', specifically, in the
    'send_project_index()' function.

    It should be used in the following manner:

    def send_project_index(req, mpr, dir):
        from adytum.plugins.trac import MultiProjectIndex

        text = MultiProjectIndex()
        text.setTitle('Available Projects')
        text.setBaseURL(mpr.idx_location)
        text.setDirs(dir)

        req.content_type = text.getContentType()
        req.write(text.render())

    '''
    def __init__(self, content_type='text/html'):
        self.content_type = content_type
        self.location = ''
        self.dirs = ''
        self.title = ''
        self.header_file = ''
        self.footer_file = ''
        self.base_url = ''
        
    def setTitle(self, title):
        self.title = title

    def setLocation(self, location):
        self.location = location

    def setBaseURL(self, url):
        self.base_url = url

    def setDirs(self, dir):
        import os
        dirs = os.listdir(dir)
        dirs.sort()
        self.dirs = dirs
        self.base_dir = dir
        default_header = 'header.tmpl'
        default_footer = 'footer.tmpl'
        self.header_file = '%s/%s' % (dir, default_header)
        self.footer_file = '%s/%s' % (dir, default_footer)        

    def getContentType(self):
        return self.content_type

    def getHeader(self):
        '''
        '''
        text = file(self.header_file, 'r').read()
        return text.replace('%%TITLE%%', self.title)

    def getProjectListing(self):
        '''
        '''
        import os
        from ConfigParser import ConfigParser

        cfg = ConfigParser()
        #heading = '<h3>%s</h3><ul>'                 # takes the project grouping as a parameter
        heading = '<ul>'
        link = '<li><a href="%s">%s</a> - %s</li>'  # takes the mpr.idx_location/location as the URL
                                                    # and project as the display link, then the desc
        footing = '</ul>'
        output = heading

        for project in self.dirs:
            proj_dir = os.path.join(self.base_dir, project)
            #output += '(debug) proj_dir: %s<br/>' % proj_dir
            proj_conf = os.path.join(proj_dir, 'conf/trac.ini')
            #output += '(debug) proj_conf: %s<br/>' % proj_conf
            if os.path.isfile(proj_conf):
                cfg.read(proj_conf)
                url = '%s/%s' % (self.base_url, project)
                output += link % (url, cfg.get('project', 'name'),
                    cfg.get('project', 'descr')
                )
        output += footing
        return output

    def getFooter(self):
        '''
        '''
        return file(self.footer_file, 'r').read()

    def render(self):
        '''
        '''
        output = self.getHeader()
        output += self.getProjectListing()
        output += self.getFooter()
        return output
