import re

class ASCIIRotation(object):
    def __init__(self, amount, direction=+1):
        self.letters = re.compile('[A-Za-z]')
        self.amount = amount
        self.direction = direction
        self.lowerBound = ord('a')
        self.upperBound = ord('z')
        self.distance = self.upperBound - self.lowerBound
    
    def rotate(self, char):
        if not re.match(self.letters, char):
            return char
        if self.direction > 0:
            char = ord(char) + (self.amount - 1)
        else:
            char = ord(char) - (self.amount + 1)
        asciiCode = (char - self.lowerBound + 1) % 26 + self.lowerBound
        return chr(asciiCode)

def rotateASCII(char, amount, direction=+1):
    """
    >>> rotateASCII('a', 0)
    'a'
    >>> rotateASCII('a', 1)
    'b'
    >>> rotateASCII('a', 1, +1)
    'b'
    >>> rotateASCII('a', 2)
    'c'
    >>> rotateASCII('a', 3)
    'd'
    >>> rotateASCII('a', 1, -1)
    'z'
    >>> rotateASCII('z', 1)
    'a'
    >>> rotateASCII('z', 1, -1)
    'y'
    >>> rotateASCII('z', 2, -1)
    'x'
    >>> rotateASCII('z', 3, -1)
    'w'
    >>> rotateASCII('a', 13) == 'a'.encode('rot13')
    True
    >>> rotateASCII('a', 26) == 'a'
    True
    >>> rotateASCII('a', 26, -1) == 'a'
    """
    ra = ASCIIRotation(amount, direction)
    return ra.rotate(char)

def rot(data, amount, direction=+1):
    """
    >>> rot('this is a test', 0)
    """
    output = ''
    ra = ASCIIRotation(amount, direction)
    for char in data.lower():
        output += ra.rotate(char)
    return output

def _getPattern(signedInt):
    direction = +1
    if signedInt < 0:
        direction = -1
    return (abs(signedInt), direction)

def rotPattern(data, patternList):
    """
    >>> rotPattern('dorks', [19, -6, 2, 9, 6])
    >>> rotPattern('this is a test', [1, 2, 3, -1, -2, -3])
    """
    output = ''
    dataLen = float(len(data))
    patternLen = float(len(patternList))
    if dataLen > patternLen:
        modifier = int(dataLen/patternLen) + 1
        patternList = patternList * modifier
    for char, pattern in zip(data, patternList):
        params = (char,) + _getPattern(pattern)
        output += rot(*params)
    return output

def _test():
    import doctest, crypto
    doctest.testmod(crypto)

if __name__ == '__main__':
    _test()
