import os
import pickle
from datetime import datetime
from StringIO import StringIO

from elementtidy import TidyHTMLTreeBuilder
from elementtree.ElementTree import Element, SubElement, ElementTree, parse

from zope.testbrowser.browser import Browser

url = 'https://www.feedburner.com/fb/a/login'
user = 'oubiwann'
passwd = open('/etc/feedburnerPasswd.txt').read().strip()
XHTML = "{http://www.w3.org/1999/xhtml}"
store = 'data.pickle'
baseURL = 'http://oubiwann.blogspot.com'
timeZone = datetime.now().strftime('%Z') or 'CST'
dateFormat = '%a, %d %b %Y %H:%M:%S ' + timeZone
feedFile = 'feed.rss'

def getData():
    # login
    b = Browser(url)
    b.getControl(name='userId').value = user
    b.getControl(name='password').value = passwd
    b.getForm().submit()

    # go to the right page
    b.getLink('Electric Duncan').click()

    # parse the page, getting rid of any namespaces
    file = StringIO(b.contents)
    tree = TidyHTMLTreeBuilder.parse(file)
    for elem in tree.getiterator():
        if elem.tag.startswith(XHTML):
            elem.tag = elem.tag[len(XHTML):]

    table = tree.findall('.//table[@id="items"]').next()

    # the first two rows are the header and totals rows, so we skip those
    newData = {}
    for row in table[2:]:
        title = row[0].find('.//img').tail.strip()
        views = int(row[1].text)
        clicks = int(row[2].text)
        newData[title] = {'views': views, 'clicks': clicks}
    return newData

def doReport(newData, oldData):
    updates = []
    for title, data in newData.items():
        update = ''
        if oldData.has_key(title):
            oldViews = oldData[title]['views']
            oldClicks = oldData[title]['clicks']
            viewCheck = data['views'] == oldViews
            clickCheck = data['clicks'] == oldClicks
            if not (viewCheck and clickCheck):
                update = "Updated data for %s: %s/%s (views/clicks; %s/%s)" % (
                    title, data['views'], data['clicks'],
                    data['views'] - oldViews, data['clicks'] - oldClicks)
                print update
        else:
            update = "New data for '%s': %s/%s (views/clicks)" % (
                title, data['views'], data['clicks'])
            print update
        updates.append((title, update))
    return updates

def getOldFeedItems():
    if not os.path.exists(feedFile):
        return []
    tree = parse(feedFile)
    root = tree.getroot()
    return root.findall('.//item')

def createFeed(entries):
    root = Element('rss', {'version':'2.0'})
    channel = SubElement(root, 'channel')
    SubElement(channel, 'title').text = "Feedburner Updates for 'Electric Duncan'"
    SubElement(channel, 'link').text = baseURL
    SubElement(channel, 'description').text = "Feedburner Updates for 'Electric Duncan'"
    date = datetime.now().strftime(dateFormat)
    SubElement(channel, 'pubDate').text = date
    SubElement(channel, 'lastBuildDate').text = date
    SubElement(channel, 'ttl').text = '1'
    for item in getOldFeedItems():
        channel.append(item)
    for entry in entries:
        item = SubElement(channel, 'item')
        title = entry[0]
        SubElement(item, 'title').text = title
        SubElement(item, 'description').text = entry[1]
        SubElement(item, 'pubDate').text = datetime.now().strftime(dateFormat)
        datepath = datetime.now().strftime('%Y/%m/%d/%H%/%M/%S')
        SubElement(item, 'guid').text = '%s/feedburner/%s/%s' % (
            baseURL, datepath, title)
    feed = ElementTree(root)
    feed.write(feedFile)

# get the old data and report on any changes
def run():
    newData = getData()
    changed = False
    try:
        fh = open(store)
        oldData = pickle.load(fh)
        if oldData != newData:
            changed = True
            updates = doReport(newData, oldData)
            createFeed(updates)
        else:
            print "Nothing changed and nothing to report."
        fh.close()
    except IOError:
        # no old data
        changed = True

    if changed:
        pickle.dump(newData, open(store, 'w+'))

if __name__ == '__main__':
    run()
