import re
import urllib2
from urllib2 import urlopen
from time import sleep
from datetime import datetime

from elementtree.ElementTree import parse

import feedparser

from adytum.util.credfile import getPickleCreds
from adytum.util.google.spreadsheet import Spreadsheet

delay = 10

def getPlayerURLs():
    playerSheet = Spreadsheet('googledocs_creds', 'Blendix Stats', 'players')
    return dict(zip(
        playerSheet.getColumnValues('Name'),
        playerSheet.getColumnValues('URL')))

def getPlayerSearches():
    playerSheet = Spreadsheet('googledocs_creds', 'Blendix Stats', 'players')
    return dict(zip(
        playerSheet.getColumnValues('Name'),
        playerSheet.getColumnValues('Search Term')))

def updateSheet(sheet, startRow, name, value):
    print name, value
    print "Updating spreadsheet ..."
    sheet.updateCell(startRow, name, str(value))
    try:
        prev = int(sheet.getCellValue(startRow - 1, name) or 0)
        sheet.updateCell(startRow, name + ' Change', str(int(value) - prev))
    except ValueError:
        # if it can't get an int, it's probably got the header, so just pass
        pass

def insertDeliciousData(players, now):
    dataSheet = Spreadsheet('googledocs_creds', 'Blendix Stats', 'del.icio.us')
    emptyRow = dataSheet.getFirstEmptyRow()

    passwdMgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
    baseURL = 'https://api.del.icio.us/v1'
    username, passwd = getPickleCreds('yahoo_creds')
    passwdMgr.add_password(None, baseURL, username, passwd)
    handler = urllib2.HTTPBasicAuthHandler(passwdMgr)
    opener = urllib2.build_opener(handler)
    urllib2.install_opener(opener)

    def getPrevious():
        dataSheet.getCellValue(emptyRow - 1, name)
    for name, url in players.items():
        print 'Pausing before next fetch, per del.icio.us API instructions ...'
        sleep(delay)
        page = urlopen('https://api.del.icio.us/v1/posts/get?url='+url)
        try:
            post = parse(page).getroot()[0]
        except IndexError:
            print "Received no posts from '%s'; skipping ..." % name
            continue
        if post.get('href') != url:
            print 'Oops! Skipping ...'
            continue
        else:
            count = post.get('others')
            updateSheet(dataSheet, emptyRow, name, count)
    dataSheet.updateCell(emptyRow, 'Date', now)

def insertBlogSearchData(players, now):
    dataSheet = Spreadsheet('googledocs_creds', 'Blendix Stats',
                            'blogsearch.google')
    emptyRow = dataSheet.getFirstEmptyRow()

    host = 'http://blogsearch.google.com/blogsearch_feeds?'
    query = 'hl=en&q=%s&as_drrb=q&as_qdr=d&ie=utf-8&num=100&output=atom'
    for name, search in players.items():
        url = host + query % search.replace(' ', '+')
        data = feedparser.parse(url)
        searchHits = data.feed.subtitle
        try:
            count = int(re.search('.*: <b>(\d+)</b>.*', searchHits).groups()[0])
        except AttributeError:
            print "Got no results for '%s' (%s)." % (name, searchHits)
            count = 0
        updateSheet(dataSheet, emptyRow, name, count)
    dataSheet.updateCell(emptyRow, 'Date', now)

def updateData():
    playerURLs = getPlayerURLs()
    playerSearchTerms = getPlayerSearches()
    now = datetime.now().strftime('%Y/%m/%d %H:%M:%S')
    insertDeliciousData(playerURLs, now)
    insertBlogSearchData(playerSearchTerms, now)

if __name__ == '__main__':
    updateData()
