import re

import gdata.spreadsheet.service
from gdata.service import RequestError

from adytum.util.credfile import getPickleCreds

"""
import gdata.spreadsheet.service

# log in
gd = gdata.spreadsheet.service.SpreadsheetsService()
gd.email, gd.password = getPickleCreds('googledocs_creds')
gd.source = 'Blendix Reports'
gd.ProgrammaticLogin()

# get spreadsheet
feed = gd.GetSpreadsheetsFeed()
[x.title.text for x in feed.entry]
['Blendix Stats', 'libraryspreadsheet', 'test1', 'DD-WRT Supported Hardware', 'PyCon 2008 Schedule']
feed.entry[0].id.text
'http://spreadsheets.google.com/feeds/spreadsheets/private/full/o02607530352343519246.1763204112004681323'
docID = feed.entry[0].id.text.split('/')[-1]

# get worksheet
feed = gd.GetWorksheetsFeed(docID)
[x.title.text for x in feed.entry]
['del.icio.us', 'users']
sheetID = feed.entry[0].id.text.split('/')[-1]

# get the number of columns
feed.total_results.text
'16'

# get column headers
feed = gd.GetCellsFeed(docID, sheetID)
[x.title.text for x in feed.entry]
['A1', 'B1', 'C1', 'D1', 'E1', 'F1', 'G1', 'H1', 'I1', 'J1', 'K1', 'L1', 'M1', 'N1', 'O1', 'P1']

[x.content.text for x in feed.entry]
['Blendix', 'Blendix Change', 'Spock', 'Spock Change', 'Spokeo', 'Spokeo Change', 'claimID', 'claimID Change', 'soup.io', 'soup.io Change', 'Tumblr', 'Tumblr Change', 'Plaxo', 'Plaxo Change', 'FriendFeed', 'FriendFeed Change']

# get the last row number to have data inserted into it

# get the row after that

# insert data
gd.UpdateCell(row=2, col=1, inputValue='this is a test', key=docID, wksht_id=sheetID)
gd.UpdateCell(row=3, col='C', inputValue='test 2', key=docID, wksht_id=sheetID)
"""
class Error(Exception):
    pass

class WorksheetNotFound(Error):
    pass

class Spreadsheet(object):

    def __init__(self, credFile, doc='', sheet='', agent=''):
        self.credFile = credFile
        if not agent:
            agent = 'AdytumSolutions, Inc. Spreadsheet Client'
        self.login(agent)
        if (doc and sheet):
            self.open(doc, sheet)
        else:
            self.doc = self.docID = ''
            self.sheet = self.sheetID = ''
        self.cellFeed = None

    def _getIDForName(self, feed, name):
        for index, entry in enumerate(feed.entry):
            if entry.title.text == name:
                return feed.entry[index].id.text.split('/')[-1]

    def _getColumnForName(self, name):
        if isinstance(name, int):
            return name
        feed = self.gd.GetCellsFeed(self.docID, self.sheetID)
        for column, entry in enumerate(feed.entry):
            if name == entry.title.text or name == entry.content.text:
                return column + 1
        raise ValueError, 'No such column name.'

    def login(self, credFile='None', agent=''):
        if not self.credFile:
            self.credFile = credFile
        gd = gdata.spreadsheet.service.SpreadsheetsService()
        gd.email, gd.password = getPickleCreds(self.credFile)
        gd.source = agent
        gd.ProgrammaticLogin()
        self.gd = gd

    def setDocID(self, doc=''):
        if doc:
            self.doc = doc
        feed = self.gd.GetSpreadsheetsFeed()
        self.docID = self._getIDForName(feed, self.doc)

    def setSheetID(self, sheet=''):
        if sheet:
            self.sheet = sheet
        try:
            feed = self.gd.GetWorksheetsFeed(self.docID)
        except RequestError:
            msg = "Could not find sheet '%s' in doc '%s' (%s)." % (
                self.sheet, self.doc, self.docID)
            raise WorksheetNotFound, msg
        self.sheetID = self._getIDForName(feed, self.sheet)

    def open(self, doc='', sheet='', docID='', sheetID=''):
        if (docID and sheetID):
            self.docID = docID
            self.sheetID = sheetID
            return
        if not (doc or sheet):
            raise ValueError, ('You must provide a value for both the doc ' +
                               'and the sheet.')
        self.setDocID(doc)
        self.setSheetID(sheet)
        self.cellFeed = None

    def refresh(self):
        self.login(self.doc)
        self.open(docID=self.docID, sheetID=self.sheetID)

    def getCells(self):
        if not self.cellFeed:
            self.cellFeed = self.gd.GetCellsFeed(self.docID, self.sheetID)
        return self.cellFeed.entry

    def getHeaders(self):
        return [x.content.text for x in self.getCells()
                if re.sub('\D+', '', x.title.text) == '1']

    def getFirstEmptyRow(self):
        last = self.getCells()[-1].title.text
        row = int(re.sub('\D+', '', last)) + 1
        return row

    def getColumnValues(self, column, skipHeader=True):
        column = self._getColumnForName(column)
        values = []
        for entry in self.getCells():
            if int(entry.cell.col) == column:
                values.append(entry.content.text)
        if skipHeader:
            return values[1:]
        return values

    def updateCell(self, row, column, value):
        column = self._getColumnForName(column)
        entry = self.gd.UpdateCell(row=row, col=column, inputValue=value,
                                   key=self.docID, wksht_id=self.sheetID)

    def getCellValue(self, row, column):
        column = self._getColumnForName(column)
        for entry in self.getCells():
            if (int(entry.cell.col) == column and
                int(entry.cell.row) == row):
                return entry.content.text


