from adytum.util.launchpad import base
from adytum.util.browser import getMatchingLink

def getNewsForm(b, projectName):
    """
    This function gets the browser object representing the form, starting from
    the logged-in (home) page.
    """
    linkText = 'Make a new announcement...'
    getMatchingLink(b, linkText, '/%s/+announce' % projectName).click()
    return b

def submitNews(b, headline, summary, url, date=None):
    """
    The function operates on the browser object representing the launchpad news
    form, pushing to data to launchpad.
    """
    dateField = 'field.publication_date.announcement_date'
    b.getControl(name='field.title').value = headline
    b.getControl(name='field.summary').value = summary
    b.getControl(name='field.url').value = url
    radioList = b.getControl(name='field.publication_date.action')
    if date:
        b.getControl(name=dateField).value = date
        radioList.getControl(value='specific').selected = True
        b.getControl(name='field.actions.announce').click()
    else:
        radioList.getControl(value='immediately').selected = True
    return b

def postNews(projectName, credFile, headline, summary, url, date=None):
    """
    This function manages the login, gets the form and submits the news data to
    the form.
    """
    loginURL = 'https://launchpad.net/pymon/+login'
    home = base.login(loginURL, credFile=credFile)
    # XXX launchpad has some issues with page load times, so if we start seeing
    # errors, we might need to implement something that does several tries for
    # getting the news form.
    form = getNewsForm(home, projectName)
    return submitNews(form, headline, summary, url, date)
