from textwrap import TextWrapper

def formatRFC822LongFieldLines(text):
    """
    This will wrap text and prepend spaces to wrapped text in accordance with
    RFC 822. We also insert line feeds at the end of lines for convenience when
    assembling into displayed text.

    >>> lines = formatRFC822LongFieldLines('a short field')
    >>> [x[:-1] for x in lines] # remove the new lines to prevent the doctest parser from barfing
    ['a short field']

    >>> lines = formatRFC822LongFieldLines('a long field ' * 20)
    >>> [x[:-1] for x in lines] # same deal as above
    ['a long field a long field a long field a long field a long field a long', '    field a long field a long field a long field a long field a long field a', '    long field a long field a long field a long field a long field a long', '    field a long field a long field a long field']

    """
    tw = TextWrapper()
    tw.width = 76
    tw.subsequent_indent = ' '*4
    return [x+'\n' for x in tw.wrap(text)]

def formatRFC822LongField(text):
    """
    A convenience function that returns the data as assembled text instead of
    lines.

    >>> print formatRFC822LongField('a short field')
    a short field
    <BLANKLINE>
    >>> print formatRFC822LongField('a long field ' * 20)
    a long field a long field a long field a long field a long field a long
        field a long field a long field a long field a long field a long field a
        long field a long field a long field a long field a long field a long
        field a long field a long field a long field
    <BLANKLINE>
    """
    return ''.join(formatRFC822LongFieldLines(text))

def _test():
    import doctest
    doctest.testmod()

if __name__ == '__main__':
    _test()
