'''

UNIX utilities.
'''
def splitSymbolic(symbolic):
    """
    >>> fileType, userPerms, groupPerms, worldPerms = splitSymbolic('-rw-r--r--')
    >>> fileType, userPerms, groupPerms, worldPerms
    ('-', 'rw-', 'r--', 'r--')

    >>> fileType, userPerms, groupPerms, worldPerms = splitSymbolic('drwxr-xr-x')
    >>> fileType, userPerms, groupPerms, worldPerms
    ('d', 'rwx', 'r-x', 'r-x')

    >>> fileType, userPerms, groupPerms, worldPerms = splitSymbolic('crw--w----')
    >>> fileType, userPerms, groupPerms, worldPerms
    ('c', 'rw-', '-w-', '---')
    """
    fileType = symbolic[0]
    user = symbolic[1:4]
    group = symbolic[4:7]
    others = symbolic[7:10]
    return (fileType, user, group, others)

def getOctalDigit(symbolic):
    """
    >>> getOctalDigit('r--')
    '4'
    >>> getOctalDigit('rw-')
    '6'
    >>> getOctalDigit('rwx')
    '7'
    >>> getOctalDigit('-w-')
    '2'
    """
    total = 0
    r, w, x = symbolic
    if not r == '-':
        total += 4
    if not w == '-':
        total += 2
    if x not in ['-', 'T', 'S']:
        total += 1
    return str(total)

def getOctalAdditional(symbolic):
    """
    This gets the sticky bit for the perms.

    >>> getOctalAdditional('-rw-r--r--')
    '0'
    >>> getOctalAdditional('-rwsr-Sr-x')
    '6'
    """
    fileType, owner, group, others = splitSymbolic(symbolic)
    perms = (owner, group, others)
    checks = (['s', 'S'], ['s', 'S'], ['t', 'T'])
    amounts = (4, 2, 1)
    add = 0
    for perm, check, amount in zip(perms, checks, amounts):
        r, w, x = perm
        if x in check:
            add += amount
    return str(add)

def getOctalPerms(symbolic):
    """
    >>> getOctalPerms('-rw-r--r--')
    '0644'
    >>> getOctalPerms('-rwsr-Sr-x')
    '6745'
    """
    fileType, owner, group, others = splitSymbolic(symbolic)
    total = ''
    for perm in (owner, group, others):
        total = total + getOctalDigit(perm)
    total = getOctalAdditional(symbolic) + total
    return total

def getSymbolicPerms(octal):
    """
    >>> getSymbolicPerms('755')
    >>> getSymbolicPerms('0644')
    '-rw-r--r--'
    >>> getSymbolicPerms('6745')
    '-rwsr-Sr-x'
    """

def _test():
    import doctest, unix
    doctest.testmod(unix)

if __name__ == '__main__':
    _test()


